/* 
 * File: utils.h
 * Author: S. Chandrasekaran
 * Date: 4.19.2014
 * Purpose: common stuff
 *
 * $Id$
 *
 */


/* *** *** */


#include <stdlib.h>
#include <stdio.h>


/* Functor style. #include this file */

#ifndef NS
#error "Namespace macro NS needs to be defined."
#endif

#ifndef DTYPE
#error "DTYPE -- type of components macro must be defined."
#endif

#ifndef ABS
#error "ABS macro needed for DTYPE"
#endif

/* *** *** */

/* Random Real array; useful for testing. Returns array in interval
   [-1,1) using drand48 */

inline void NS(randArray) (DTYPE * const restrict arr, const size_t N) {
  size_t i;
  for (i = 0; i < N; i++) arr[i] = (DTYPE) (2 * drand48 () - 1);
  return;
};


/* *** *** */


/* Constant Real array; useful for testing */

inline void NS(constArray) (DTYPE * const restrict arr,
			    const size_t N, const DTYPE v) {
  size_t i;
  for (i = 0; i < N; i++) arr[i] = v;
  return;
};


/* *** *** */


/* Range Real array; useful for testing */

inline void NS(rangeArray) (DTYPE * const restrict arr, const size_t N,
			    const DTYPE start, const DTYPE incr) {
  size_t i;
  DTYPE v;
  for (i = 0, v = start; i < N; i++, v += incr) arr[i] = v;
  return;
};


/* *** *** */

/* X <- alpha * X + beta * Y */

inline void NS(linComb) (const DTYPE alpha, DTYPE * const restrict X,
			 const DTYPE beta, const DTYPE * const restrict Y,
			 const size_t N) {
  size_t i;
  for (i = 0; i < N; i++) X[i] = alpha * X[i] + beta * Y[i];
  return;
};


/* *** *** */

/* Scale array */

inline void NS(scaleArray) (const DTYPE alpha, DTYPE * const restrict X,
			    const size_t N) {
  size_t i;
  for (i = 0; i < N; i++) X[i] *= alpha;
  return;
};


/* *** *** */


/* Maximum absolute value in array */

inline DTYPE NS(maxVal) (const DTYPE * const restrict x, const size_t N) {
  DTYPE v; size_t i;
  for (i = 0, v = 0; i < N; i++) {
    DTYPE xi = ABS (x[i]);
    if (v < xi) v = xi;
  };
  return v;
};


/* *** *** */


/* Fill with I */

inline void NS(eyeMatrix) (DTYPE * const restrict arr,
			   const size_t M, const size_t N) {
  constArray (arr, N * M, 0);
  { size_t j;
    for (j = 0; j < (M <= N ? M : N); j++) arr[j*(1+M)] = 1;
  }
  return;
};


/* *** *** */


inline void NS(printArray) (const DTYPE * const restrict arr, size_t N) {
  size_t i;
  for (i = 0; i < N; i++) printf ("%g ", arr[i]);
};

/* *** *** */

inline void NS(printMatrix) (const DTYPE * const restrict arr,
			     const size_t M, const size_t N) {
  size_t i;
  for (i = 0; i < M; i++) {
    size_t j;
    for (j = 0; j < N; j++) printf ("%g ", arr[j*M+i]);
    printf ("\n");
  };
  return;
};

/* *** *** */


/* Scale column [j] of matrix [arr] having [N] rows by number [s] */
inline void NS(scaleColumn) (const size_t N, Real * const restrict arr,
			     const size_t j, Real s) {
  size_t i;
  for (i = 0; i < N; i++) arr[j*N+i] *= s;
  return;
};


/* *** *** */

/* Shift 2D array */

/* 
 * shift -- N x 1 array of
 * arr -- N x M array changed arr[:,i] + shift
 * arr and shift cannot overlap
 */
inline void NS(shiftMatrix) (const Real * const restrict shift,
			     Real * const restrict arr,
			     const size_t N, const size_t M) {
  size_t i;
  for (i = 0; i < M; i++) {
    size_t j;
    for (j = 0; j < N; j++) arr[i*N+j] += shift[j];
  };

  return;
};


/* *** *** */

